/*
 * JDBReport Generator
 * 
 * Copyright (C) 2006-2008 Andrey Kholmanskih. All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, write to the 
 *
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330,
 * Boston, MA  USA  02111-1307
 * 
 * Andrey Kholmanskih
 * support@jdbreport.com
 */
package jdbreport.source;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.naming.NamingException;

import and.dbcomp.DbSource;

/**
 * @version 1.1 03/09/08
 * @author Andrey Kholmanskih
 * 
 */
public class JdbcReportSource extends DbSource implements
		Iterable<JdbcDataSet>, Cloneable {

	private static final long serialVersionUID = 1L;
	private List<JdbcDataSet> dsList;
	private Connection tmpConnection;

	public JdbcReportSource() {
		this("Default");
	}

	public JdbcReportSource(String name) {
		super();
		setAlias(name);
		setAutoCommit(false);
		dsList = new ArrayList<JdbcDataSet>();
	}

	public Connection getConnection() throws SQLException,
			InstantiationException, IllegalAccessException,
			ClassNotFoundException, NamingException {
		if (tmpConnection != null) {
			return tmpConnection;
		}
		return super.getConnection();
	}

	public void setConnection(Connection connection) {
		this.tmpConnection = connection;
		super.setConnection(null);
	}

	public Object clone() {
		JdbcReportSource source = null;
		try {
			source = (JdbcReportSource) super.clone();
			source.setConnection(null);
			source.dsList = new ArrayList<JdbcDataSet>();
			for (JdbcDataSet ds : dsList) {
				JdbcDataSet newDs = (JdbcDataSet) ds.clone();
				newDs.setSource(source);
				source.dsList.add(newDs);
			}
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return source;
	}

	public int getDataSetCount() {
		return dsList.size();
	}

	public void add(JdbcDataSet ds) {
		ds.setSource(this);
		dsList.add(ds);
	}

	public JdbcDataSet remove(int index) {
		JdbcDataSet ds = (JdbcDataSet) dsList.remove(index);
		ds.setSource(null);
		return ds;
	}

	public void remove(JdbcDataSet ds) {
		ds.setSource(null);
		dsList.remove(ds);
	}

	public JdbcDataSet getDataSet(int index) {
		return (JdbcDataSet) dsList.get(index);
	}

	public Iterator<JdbcDataSet> iterator() {
		return dsList.iterator();
	}

	public List<JdbcDataSet> getDataSetList() {
		return dsList;
	}
}
